import click

from service_buddy.context.service_context import ServiceContext
from service_buddy.util.log_handler import configure_logging


@click.group()
@click.option("--application-filter", envvar='FILTER', help='Constrain command to operate on applications'
                                                            ' that match the passed filter')
@click.option("--service-filter", envvar='SERVICE_FILTER', help='Constrain command to operate on services'
                                                            ' that match the passed filter')
@click.option("--service-directory", envvar='SERVICE_DIRECTORY', type=click.Path(),
              default="./services",
              help='Directory containing service definitions in <app>/service.json format.  Default is \'./services\'')
@click.option('--destination-directory',
              envvar='DESTINATION',
              type=click.Path(),
              default="./code",
              help='The directory where the repositories for each service should be created or currently exist.')
@click.option("--verbose", is_flag=True, help='Print verbose status messages')
@click.option("--dry-run", is_flag=True, help='Preview effect of action')
@click.pass_context
def cli(ctx, application_filter, service_filter, service_directory, destination_directory, verbose, dry_run):
    # type: (object, str,str, str ,bool, bool) -> None
    """
    CLI for managing the repositories and build pipeline in a micro-service architecture..
    """
    configure_logging(verbose)
    ctx.obj = ServiceContext(
        filter_string=application_filter,
        service_filter_string=service_filter,
        service_directory=service_directory,
        destination_directory=destination_directory,
        dry_run=dry_run
    )


# noinspection PyUnresolvedReferences
from service_buddy.commands.clone import command
# noinspection PyUnresolvedReferences
from service_buddy.commands.git import command
# noinspection PyUnresolvedReferences
from service_buddy.commands.initialize import command
# noinspection PyUnresolvedReferences
from service_buddy.commands.list import command
# noinspection PyUnresolvedReferences
from service_buddy.commands.bootstrap import command




import json
import os

import logging
from cookiecutter.main import cookiecutter


def _make_cookie_safe(service_definition):
    ret = {}
    for key, valey in service_definition.iteritems():
        ret[key.replace('-', "_")] = valey
    return ret


class CookeCutterProjectCreator(object):
    def __init__(self, template_dir, dry_run, templates):
        super(CookeCutterProjectCreator, self).__init__()
        self.template_dir = template_dir
        self.templates =templates
        self.dry_run = dry_run

    def create_project(self, service_definition, app_dir,extra_config=None):
        template = self._lookup_service_template(service_definition.get_service_type())
        if template['type'] == 'file':
            location = os.path.abspath(os.path.join(self.template_dir, template['location']))
        else:
            location = template['location']
        extra_context = _make_cookie_safe(service_definition)
        if extra_config:
            extra_context.update(_make_cookie_safe(extra_config))
        if self.dry_run:
            logging.error("Creating project from template {} ".format(location))
        else:
            return cookiecutter(location, no_input=True, extra_context=extra_context, output_dir=app_dir)

    def _lookup_service_template(self, service_type):
        if service_type not in self.templates:
            raise Exception("Unknown code template - {}".format(service_type))
        return self.templates[service_type]

    @classmethod
    def get_type(cls):
        return "cookiecutter"




import click
from service_buddy.commandline import cli

from service_buddy.context.service_context import ServiceContext


@cli.command(name='git', short_help="Run arbitrary git command for each service")
@click.argument('cmd', nargs=-1)
@click.pass_obj
def git_exec(service_ctx, cmd):
    # type: (ServiceContext, str) -> None
    """
    :param cmd: The git command to execute in each service directory.
    This can be used for commands such as "--git pull" or "--git commit -m 'Big old commit'"
    """
    service_ctx.vcs.validate_repositories(service_ctx.application_map)
    service_ctx.vcs.git_exec(application_map=service_ctx.application_map,
                             destination_directory=service_ctx.destination_directory,
                             args=cmd)



import os

import logging

from service_buddy.service import loader
from service_buddy.vcs.vcs import VCS


class ServiceContext(object):

    def __init__(self, filter_string, service_filter_string, service_directory, destination_directory, dry_run):
        super(ServiceContext, self).__init__()
        self.destination_directory = destination_directory
        self.dry_run = dry_run
        if not os.path.exists(service_directory):
            logging.warn("Service directory does not exist, not loading services or vcs - {}".format(service_directory))
        else:
            self.application_map = loader.load_service_definitions(service_directory=service_directory, app_filter=filter_string, service_filter=service_filter_string)
            self.vcs = VCS(service_directory=service_directory, dry_run=dry_run)



import json
import logging
import os
from collections import OrderedDict

from service_buddy.util.command_util import invoke_process


class BambooBuildCreator(object):

    def __init__(self):
        self.build_system_url = 'url'

    def options(self):
        opt_dict = OrderedDict()
        opt_dict[self.build_system_url]="URL of bamboo server"
        opt_dict['user']="Username for authentication when creating builds"
        opt_dict['password']="Password for authentication when creating builds"
        return opt_dict

    def init(self, dry_run, default_config, build_templates,user=None, password=None):
        self.dry_run = dry_run
        # use bamboo-url as backup
        self.url = default_config.get(self.build_system_url,default_config.get('bamboo-url',None))
        self.build_templates = build_templates
        if user and password:
            with open('.credentials','w') as cred_file:
                cred_file.writelines('username={}'.format(user))
                cred_file.writelines('password={}'.format(password))
                cred_file.flush()

    def create_project(self, service_definition, app_dir):
        logging.info("Creating bamboo build")
        if service_definition.get_service_type() not in self.build_templates:
            raise Exception("Build template not found for service type {}".format(service_definition.get_service_type()))

        build_template = self.build_templates.get(service_definition.get_service_type())['type']
        args = [
            'java',
            '-Dbamboo.specs.log.level=DEBUG',
            '-jar',
            'bamboo-plan-1.0-SNAPSHOT.jar',
            '--build-template', build_template,
            '--bamboo-url', self.url,
            '--application', service_definition.get_app(),
            '--role', service_definition.get_role()
        ]

        res = invoke_process(args, dry_run=self.dry_run)
        if res > 0:
            raise Exception("Error creating bamboo build")
        else:
            logging.info("Done creating bamboo build")

    @classmethod
    def get_type(cls):
        return "bamboo"



import logging
import subprocess

from service_buddy.util.log_handler import print_red_bold


def invoke_process(args, exec_dir=None, dry_run=False):
    if dry_run:
        print_red_bold(u"\t {}".format(str(args)))
        return 0
    else:
        arg_list = {'args': args}
        arg_list['stderr'] = subprocess.STDOUT
        if exec_dir:
            arg_list['cwd'] = exec_dir

        logging.info(u'[exec] invoke_process args=%r, exec_dir=%r, dry_run=%r', args, exec_dir, dry_run)
        returncode = 0
        try:
            output = subprocess.check_output(**arg_list)
        except subprocess.CalledProcessError, e:
            output = ""
            returncode = e.returncode

        for line in output.splitlines():
            logging.info(u'[exec] %s', line)
        logging.info(u'[exec] invoke_process complete args=%r, exec_dir=%r, dry_run=%r', args, exec_dir, dry_run)

        return returncode



import json
import os

import logging

from service_buddy.service.service import Service
from service_buddy.util.command_util import invoke_process


class TravisBuildCreator(object):

    def options(self):
        return {}

    def init(self, dry_run, default_config, build_templates,user=None, password=None):
        self.dry_run = dry_run
        self.build_templates = build_templates
        self.build_configuration = default_config.get('build-configuration',{})
        self.use_org = default_config.get("use-travis-open-source", True)
        self.gh_token = default_config.get('github-token', os.environ.get('GITHUB_TOKEN',''))
        self.pypi_user = default_config.get('pypi-user', os.environ.get('PYPI_USER'))
        self.pypi_pass = default_config.get('pypi-pass', os.environ.get('PYPI_PASS'))
        self.default_args = ['--no-interactive']
        if self._invoke_travis(['--help'],append_org=False) != 0:
            raise Exception("Travis CLI must be installed to use travis build creator.  "
                            "See https://github.com/travis-ci/travis.rb#installation")
        if self._invoke_travis([ 'login', '--github-token', self.gh_token]) != 0:
            raise Exception("Unable to authenticate Travis CLI. Please verify github token is configured properly.")

    def create_project(self, service_definition, app_dir):
        # type: (Service, str) -> None
        if service_definition.get_service_type() not in self.build_templates:
            raise Exception(
                "Build template not found for service type {}".format(service_definition.get_service_type()))
        else:
            build_type = self.build_templates.get(service_definition.get_service_type())['type']
        service_dir = service_definition.get_service_directory(app_dir=app_dir)
        if os.path.exists(self._get_travis_file(service_dir)):
            logging.warn("travis build file exists - enabling repo")
            self._invoke_travis([ 'enable'], exec_dir=service_dir)
        else:
            build_template = self.build_configuration.get(build_type,None)
            if build_template:
                self.create_build(service_dir, build_template,service_definition)
            else:
                logging.warn("Could not locate build template"
                                             " for build type - {}:{}".format(service_definition.get_service_type(),
                                                                              build_type))


    def _get_travis_file(self, service_dir):
        return os.path.join(service_dir, ".travis.yml")

    @classmethod
    def get_type(cls):
        return "travis"

    def create_build(self, service_dir, build_template, service_definition):
        language_ = build_template.get('language', 'python')
        args = [
            'init',
            language_
        ]
        if language_ == 'python':
            args.append('--python')
            args.append('2.7')

        install_script = build_template.get('install', None)
        if install_script:
            args.append('--install')
            self._append_rendered_arguments(args, install_script, service_definition)

        script = build_template.get('script', None)
        if script:
            args.append('--script')
            self._append_rendered_arguments(args, script, service_definition)

        self._invoke_travis(args, exec_dir=service_dir)
        use_pypi = build_template.get('pypi-deploy', False)
        if use_pypi:
            self._write_deploy_stanza(service_dir)
            self._invoke_travis([ 'encrypt', self.pypi_pass, '--add', 'deploy.password'], exec_dir=service_dir)
        invoke_process(['git', 'add','.travis.yml'],exec_dir=service_dir,dry_run=self.dry_run)
        invoke_process(['git', 'commit','-m','Travis build file - generated by service-buddy'],exec_dir=service_dir,dry_run=self.dry_run)

    @staticmethod
    def _append_rendered_arguments(args, install_script, service_definition):
        if isinstance(install_script, basestring):
            install_script = [install_script]
        for script in install_script:
            if "${" in script:
                # Assume is there is bash style escape we don't python escape.  sucks but the extent of my attention
                args.append("\"{}\"".format(script))
            else:
                args.append("\"{}\"".format(script.format(**service_definition)))

    def _write_deploy_stanza(self, service_dir):
        with open(self._get_travis_file(service_dir), 'a') as build_file:
            build_file.writelines([
                '\n'
                'deploy\n',
                '\tprovider: pypi\n',
                '\tuser: {}\n'.format(self.pypi_user),
                '\tdistributions: sdist bdist_wheel\n',
                '\tserver: https://upload.pypi.org/legacy\n'
            ])

    def _invoke_travis(self, args,exec_dir=None,append_org=True):
        base_args = ['travis']
        base_args.extend(args)
        base_args.extend(self.default_args)
        if append_org: base_args.append( '--org' if self.use_org else '--pro')
        if exec_dir:
            return invoke_process(args=base_args,exec_dir=exec_dir,dry_run=self.dry_run)
        else:
            return invoke_process(args=base_args,dry_run=self.dry_run)



import logging

from github import Github
from requests import HTTPError

from service_buddy.util.command_util import invoke_process


class GitHubVCSProvider(object):
    @classmethod
    def get_type(cls):
        return 'github'


    def __init__(self, ):
        super(GitHubVCSProvider, self).__init__()
        self.repo_root = ""
        self.dry_run = ""
        self.client = None

    def init(self, user, password, repo_root, dry_run):
        self.repo_root = repo_root
        self.dry_run = dry_run
        if user and password:
            self.client = Github(user, password).get_organization(self.repo_root)
        else:
            logging.warn(
                "VCS username and password not configured - assuming git executable has appropriate authorization for "
                "repo checks")
            self.client = None

    def find_repo(self, service_definition):
        fq_repository_name = "{}/{}".format(self.repo_root, service_definition.get_repository_name())
        try:
            if self.client:
                ssh_url = None
                for repo in self.client.get_repos():
                    if repo.name == fq_repository_name:
                        ssh_url = repo.ssh_url
                        break
            else:
                ssh_url = 'ssh://git@github.com/{}'.format(fq_repository_name)
                result = invoke_process(args=['git', 'ls-remote', ssh_url, '>', '/dev/null'], exec_dir=None,
                                        dry_run=self.dry_run)
                if result != 0:
                    logging.info("Could not find repository with git executable - {}".format(
                        service_definition.get_repository_name()))
                    ssh_url = None
            return ssh_url
        except HTTPError:
            logging.info("Could not find repository through github API - {}".format(service_definition.get_repository_name()))

    def create_repo(self, service_defintion):
        # name, description=github.GithubObject.NotSet,
        #  private=github.GithubObject.NotSet,
        # has_issues=github.GithubObject.NotSet,
        # has_wiki=github.GithubObject.NotSet,
        # has_downloads=github.GithubObject.NotSet,
        # team_id=github.GithubObject.NotSet,
        # gitignore_template=github.GithubObject.NotSet
        payload = {
            "name": service_defintion.get_fully_qualified_service_name(),
            "description": service_defintion.get_description(),
            "private": True,
            "has_issues": False,
            "has_projects": False,
            "has_wiki": False
        }
        if self.dry_run:
            logging.error("Creating repo {}".format(str(payload)))
        else:
            if self.client is None:
                raise Exception("VCS pass required for create repo operation")
            repo = self.client.create_repo(**payload)
            return repo.ssh_url



import json
import os

import logging
from collections import OrderedDict

from Bitbucket import BitbucketVCSProvider
from service_buddy.service.loader import walk_service_map, safe_mkdir, ensure_app_directory_exists, \
    ensure_service_directory_exists
from service_buddy.service.service import Service
from service_buddy.util.command_util import invoke_process
from service_buddy.vcs.github_vcs import GitHubVCSProvider


vcs_provider_map = {
    BitbucketVCSProvider.get_type(): BitbucketVCSProvider(),
    GitHubVCSProvider.get_type(): GitHubVCSProvider()}
vcs_providers = [key for key in vcs_provider_map.iterkeys()]

options = OrderedDict()
options['root-user']="Organization name. team name or root user used by vcs provider"
options['user']="Username for authentication when creating repositories"
options['password']="Password for authentication when creating repositories"
class VCS(object):
    def __init__(self, service_directory, dry_run):
        super(VCS, self).__init__()
        default_path = os.path.join(service_directory, "vcs-config.json")
        if os.path.exists(default_path):
            with open(default_path) as fp:
                defaults = json.load(fp)
                self.default_provider = defaults.get('provider', None)
                self.repo_root = defaults.get('root-user', os.environ.get('VCS_ROOT_USER'))
                self.user = defaults.get('user', os.environ.get('VCS_USER'))
                self.password = defaults.get('password', os.environ.get('VCS_PASSWORD'))
        else:
            raise Exception("Could not local 'vcs-config.json' in service directory")
        self.dry_run = dry_run


        if self.default_provider not in vcs_provider_map:
            raise Exception("Requested provider is not configured {}".format(self.default_provider))
        else:
            self._get_default_vcs_provider().init(self.user, self.password, self.repo_root, dry_run)

    def _get_default_vcs_provider(self):
        return vcs_provider_map[self.default_provider]

    def validate_repositories(self, application_map):
        def populate_repo_metadata(service_definition):
            repo_url = self._get_default_vcs_provider().find_repo(service_definition)
            service_definition.set_git_url(repo_url)

        walk_service_map(application_map=application_map, application_callback=None,
                         service_callback=populate_repo_metadata)

    def create_project(self, service_definition, app_dir):
        return self.init_repo(service_definition=service_definition,
                              service_dir=service_definition.get_service_directory(app_dir))

    def init_repo(self, service_definition, service_dir):
        repo_url = self._get_default_vcs_provider().create_repo(service_definition)
        args = ['git', 'init']
        invoke_process(args, exec_dir=service_dir, dry_run=self.dry_run)
        args = ['git', 'add', '*', '**/*']
        invoke_process(args, exec_dir=service_dir, dry_run=self.dry_run)
        args = ['git', 'commit', '-m', 'Initial commit']
        invoke_process(args, exec_dir=service_dir, dry_run=self.dry_run)
        args = ['git', 'remote', 'add', 'origin', repo_url]
        invoke_process(args, exec_dir=service_dir, dry_run=self.dry_run)
        args = ['git', 'push', '-u', 'origin', 'master']
        invoke_process(args, exec_dir=service_dir, dry_run=self.dry_run)
        service_definition.set_git_url(repo_url)

    def clone_service(self, application_map, destination_directory):
        # type: (dict, str) -> None
        safe_mkdir(destination_directory)

        def clone_repository(service_defintion):
            # type: (Service) -> None
            app_dir = ensure_app_directory_exists(destination_directory, service_defintion)
            if service_defintion.repo_exists():
                clone_url = service_defintion.get_git_url()
                args = ['git', 'clone', clone_url,service_defintion.get_fully_qualified_service_name()]
                service_directory = service_defintion.get_service_directory(app_dir=app_dir)
                if os.path.exists(service_directory):
                    logging.warn("Skipping clone step directory exists - {}".format(service_directory))
                else:
                    invoke_process(args, app_dir, self.dry_run)

        walk_service_map(application_map=application_map, application_callback=None,
                         service_callback=clone_repository)

    def git_exec(self, application_map, destination_directory, args):
        # type: (dict,str, list) -> None

        def git_exec(service_defintion):
            # type: (Service) -> None
            destination_dir = ensure_service_directory_exists(destination_directory=destination_directory,
                                                              service_defintion=service_defintion,
                                                              create=False)
            if not destination_directory:
                logging.warn("Service '{}' did not exist in destination directory - {}".format(service_defintion.get_fully_qualified_service_name(),destination_directory))
                logging.warn("Skipping running git command - git {}".format(str(args)))
                return
            logging.warn("Invoking git in directory - '{}' ".format(destination_dir))
            git_args = ['git']
            git_args.extend(args)
            invoke_process(git_args, destination_dir, self.dry_run)

        walk_service_map(application_map=application_map, application_callback=None,
                         service_callback=git_exec)



import json
import logging
import os
from collections import defaultdict

from service_buddy.service.application import Application
from service_buddy.service.service import Service
import re

def load_service_definitions(service_directory, app_filter=None, service_filter=None):
    service_dir = os.path.abspath(service_directory)
    listdir = os.listdir(service_dir)
    service_map = defaultdict(dict)
    for dir in listdir:
        if _is_valid_app(dir, service_dir) and application_filter(app_filter, dir):
            service_definition_file = os.path.join(service_dir, dir, 'service.json')
            if not os.path.exists(service_definition_file):
                logging.warn("Skipping invalid application directory - no service.json exists - {}".format(service_dir))
            else:
                service_map[dir] = Application(dir)
                with open(service_definition_file) as service_def:
                    service_definitions = json.load(service_def)
                    for role, definition in service_definitions.iteritems():
                        if re.match(service_filter or ".*", role):
                            service_map[dir].add_service(
                                role, Service(app=dir, role=role, definition=definition,
                                app_reference=service_map[dir])
                            )
    return service_map


def _is_valid_app(dir, service_dir):
    return os.path.isdir(os.path.join(service_dir, dir)) and not dir.startswith('.')


def application_filter(app_filter, application):
    return app_filter is None or application.startswith(app_filter)


def walk_service_map(application_map, application_callback, service_callback):
    for application, service_map in application_map.iteritems():
        if application_callback: application_callback(application)
        for service, service_definition in service_map.iteritems():
            if service_callback: service_callback(service_definition)


def safe_mkdir(directory):
    if not os.path.exists(directory):
        os.mkdir(directory)


def ensure_app_directory_exists(destination_directory, service_defintion):
    app_dir = os.path.join(destination_directory, service_defintion.get_app())
    safe_mkdir(app_dir)
    return app_dir


def ensure_service_directory_exists(destination_directory, service_defintion, create=True):
    # type: (str, Service) -> object
    app_dir = ensure_app_directory_exists(destination_directory=destination_directory,
                                          service_defintion=service_defintion)
    service_directory = service_defintion.get_service_directory(app_dir=app_dir)
    if create:
        safe_mkdir(service_directory)
        return service_directory
    else:
        if os.path.exists(service_directory):
            return service_directory
        return None



import logging

from pybitbucket import repository
from pybitbucket.auth import BasicAuthenticator
from pybitbucket.bitbucket import Client
from pybitbucket.repository import RepositoryPayload, RepositoryForkPolicy
from requests import HTTPError

from service_buddy.util.command_util import invoke_process


class BitbucketVCSProvider(object):
    @classmethod
    def get_type(cls):
        return 'bitbucket'
    
    def __init__(self):
        super(BitbucketVCSProvider, self).__init__()
        self.repo_root = ""
        self.dry_run = ""
        self.client = None

    def init(self, user, password, repo_root,dry_run):
        self.dry_run = dry_run
        if user and password:
            self.client = Client(
                BasicAuthenticator(
                    user,
                    password,
                    'pybitbucket@mailinator.com'))
        else:
            logging.warn("VCS username and password not configured - assuming git executable has appropriate "
                         "authorization for repo checks")
            self.client = None
        self.team_root_user = repo_root
        self.bitbucket_repo = repository.Repository

    def find_repo(self, service_definition):
        fq_repository_name = "{}/{}".format(self.team_root_user, service_definition.get_repository_name())
        try:
            if self.client:
                repo = self.bitbucket_repo.find_repository_by_full_name(full_name=fq_repository_name,
                                                                    client=self.client)
                bitbucket_url = repo.clone['ssh']
            else:
                bitbucket_url = 'ssh://git@bitbucket.org/{}'.format(fq_repository_name)
                result = invoke_process(args=['git', 'ls-remote', bitbucket_url, '>', '/dev/null'], exec_dir=None, dry_run=self.dry_run)
                if result != 0:
                    logging.info("Could not find repository with git executable - %r", service_definition.get_repository_name())
                    bitbucket_url = None
            return bitbucket_url
        except HTTPError as e:
            logging.info("Could not find repository through Bitbucket API - %r (error=%r)",
                service_definition.get_repository_name(),
                e,
            )

    def create_repo(self, service_defintion):
        payload = RepositoryPayload()\
                    .add_description(str(service_defintion.get_description()))\
                    .add_is_private(True)\
                    .add_name(service_defintion.get_fully_qualified_service_name())\
                    .add_owner(self.team_root_user)\
                    .add_fork_policy(RepositoryForkPolicy.NO_PUBLIC_FORKS)
        if self.dry_run:
            logging.error("Creating repo %r", str(payload._payload))
        else:
            if self.client == None:
                raise Exception("VCS pass required for create repo operation")
            repo = self.bitbucket_repo.create(
                payload=payload,
                repository_name=service_defintion.get_fully_qualified_service_name(),
                owner=self.team_root_user,
                client=self.client )
            return repo.clone['ssh']



import os

REPO_URL = 'repository_url'

REPOSITORY_NAME = 'repository'
REPOSITORY_URL = 'repository_url'
DESCRIPTION = 'description'
ROLE = 'role'
APPLICATION = 'application'
FQN = 'project_name'
FORCE_RECREATE_BUILD = 'recreate-build'
SERVICE_TYPE = 'service-type'


class Service(dict):
    def __init__(self, app, role, definition, app_reference=None):
        super(Service, self).__init__()
        self.update(definition)
        self.app_ref = app_reference
        self[APPLICATION] = app
        self[ROLE] = role
        self[FQN] = self.get_fully_qualified_service_name()

    def get_fully_qualified_service_name(self):
        return "{application}-{role}".format(**self).replace(' ','_')

    def force_recreate_build(self):
        return self.get(FORCE_RECREATE_BUILD,os.environ.get('RECREATE_BUILDS',False))

    def get_description(self):
        return self[DESCRIPTION]

    def get_service_type(self):
        return self[SERVICE_TYPE]

    def repo_exists(self):
        return self.get(REPOSITORY_URL,None) is not None

    def get_role(self):
        return self[ROLE]

    def get_app(self):
        return self[APPLICATION]

    def get_repository_name(self):
        return self[REPOSITORY_NAME] if REPOSITORY_NAME in self else self.get_fully_qualified_service_name()

    def set_git_url(self, url):
        self[REPO_URL] = url

    def get_git_url(self):
        return self.get(REPO_URL,None)

    def get_contract_test_git_url(self):
        return self.app_ref.get_contract_test_git_url()

    def get_service_directory(self,app_dir):
        return os.path.join(app_dir,self.get_fully_qualified_service_name())

    def set_service_type(self, param):
        self[SERVICE_TYPE] = param


import json
import os
import logging
from service_buddy.ci.bamboo_build_creator import BambooBuildCreator
from service_buddy.ci.travis_build_creator import TravisBuildCreator

build_system_map = {
            BambooBuildCreator.get_type(): BambooBuildCreator(),
            TravisBuildCreator.get_type(): TravisBuildCreator()}
build_systems = [key for key in build_system_map.iterkeys()]


class BuildCreator(object):
    def __init__(self, template_directory, dry_run):
        super(BuildCreator, self).__init__()
        default_path = os.path.join(template_directory, "build-config.json")
        if os.path.exists(default_path):
            with open(default_path) as fp:
                defaults = json.load(fp)
                self.default_provider = defaults.get('provider', None)
                self.build_templates = defaults.get('build-templates', {})
                self.user = defaults.get('user', os.environ.get('BUILD_SYSTEM_USER'))
                self.password = defaults.get('password', os.environ.get('BUILD_SYSTEM_PASSWORD'))
                self.always_recreate_builds = defaults.get('build-creation-is-idempotent', True)
                self.default_config = defaults
        else:
            logging.warn("Could not local 'build-config.json' in code template directory")
            self.default_provider = "bamboo"
        if self.default_provider not in build_system_map:
            raise Exception("Requested provider is not configured {}".format(self.default_provider))
        else:
            creator = self._get_default_build_creator()
            creator.init(
                dry_run=dry_run,
                default_config=self.default_config,
                build_templates=self.build_templates,
                user=self.user, password=self.password
            )

    def _get_default_build_creator(self):
        # type: () -> BuildCreator
        return build_system_map[self.default_provider]

    def create_project(self, service_definition, app_dir, force_build_creation=False):
        # type: (Service, str) -> object
        do_create = not service_definition.repo_exists() or self.always_recreate_builds or force_build_creation
        logging.info(
            '[create project] repo exists: %r, always_recreate: %r, force create: %r do_create: %r',
            service_definition.repo_exists(), self.always_recreate_builds, force_build_creation, do_create
        )
        if do_create:
            return self._get_default_build_creator().create_project(
                service_definition=service_definition,
                app_dir=app_dir
            )



import logging

from service_buddy.service.loader import walk_service_map


def pretty_print_application(app): logging.error(u"{}".format(app))


def pretty_print_service(service_definition):
    logging.warn(u"\t {}".format(service_definition.get_role()))
    for dat in service_definition:
        secondary_indent = '\t\t' if len(dat) <= 10 else '\t'
        logging.info(u"\t\t {}{}- {}".format(dat, secondary_indent, service_definition[dat]))


def pretty_print_services(application_map):
    walk_service_map(application_map, pretty_print_application, pretty_print_service)


import logging

from service_buddy.ci.ci import BuildCreator
from service_buddy.code.code_creator import CodeCreator
from service_buddy.service.loader import safe_mkdir, walk_service_map, \
    ensure_app_directory_exists
from service_buddy.service.service import Service
from service_buddy.service.service_template_generator import ServiceTemplateGenerator
from service_buddy.util.pretty_printer import pretty_print_service


class Initializer(object):
    def __init__(self, vcs, destination_directory, dry_run, code_template_directory,
                 skip_code_creation=False,
                 skip_build_creation=False,
                 skip_git_creation=False,
                 force_build_creation=False,
                 ):
        super(Initializer, self).__init__()
        self.skip_code_creation = skip_code_creation
        self.force_build_creation = force_build_creation
        self.skip_build_creation = skip_build_creation
        self.skip_git_creation = skip_git_creation
        self.code_generator = CodeCreator(code_template_directory=code_template_directory,
                                          dry_run=dry_run)
        self.build_creator = BuildCreator(template_directory=code_template_directory,
                                          dry_run=dry_run)
        self.dry_run = dry_run
        safe_mkdir(directory=destination_directory)
        self.destination_directory = destination_directory
        self.vcs = vcs

    def init_app(self, app):
        pass

    def init_service(self, definition):
        # type: (Service ) -> None
        app_dir = ensure_app_directory_exists(self.destination_directory, service_defintion=definition)
        if not definition.repo_exists():
            logging.info("Creating Service - %r", definition.get_fully_qualified_service_name())
            if self.skip_code_creation:
                logging.info("Skipping code creation")
            else:
                logging.info("Running code creation")
                self.code_generator.create_project(definition, app_dir)

            if self.skip_git_creation:
                logging.info("Skipping git creation")
            else:
                logging.info("Running git creation")
                self.vcs.create_project(definition, app_dir)
        else:
            logging.info("Service exists - {}".format(definition.get_fully_qualified_service_name()))

        if self.skip_build_creation:
            logging.info("Skipping build creation")
        else:
            logging.info("Running build creation")
            self.build_creator.create_project(definition, app_dir, force_build_creation=self.force_build_creation)

        pretty_print_service(definition)
        logging.info("Done creating Service - %r", definition.get_fully_qualified_service_name())

    def initialize_services(self, application_map):
        walk_service_map(application_map=application_map, application_callback=self.init_app,
                         service_callback=self.init_service)



import json
import os

import logging

from copy import deepcopy

from service_buddy.service.service import Service
from service_buddy.code.cookie_cutter_creator import CookeCutterProjectCreator
from service_buddy.service.service_template_generator import ServiceTemplateGenerator


class CodeCreator(object):
    def __init__(self, code_template_directory, dry_run):
        super(CodeCreator, self).__init__()
        default_path = os.path.join(code_template_directory, "code-template-config.json")
        if os.path.exists(default_path):
            with open(default_path) as fp:
                defaults = json.load(fp)
                self.default_provider = defaults.get('provider', None)
                self.extended_templates = defaults.get('code-template-definitions', {})
        else:
            logging.info("Could not locate 'code-template-config.json' in code template directory")
            self.default_provider = "cookiecutter"
            self.extended_templates = {}
        built_in = self._load_service_templates(
            os.path.join(os.path.dirname(os.path.abspath(__file__)), 'builtin-code-templates.json'))
        built_in.update(self.extended_templates)
        self.templates = self._resolve_alias(built_in)
        self.service_template_generator = ServiceTemplateGenerator()
        self.code_creators = {
            CookeCutterProjectCreator.get_type(): CookeCutterProjectCreator(template_dir=code_template_directory,
                                                                            dry_run=dry_run,
                                                                            templates=self.templates)}
        if self.default_provider not in self.code_creators:
            raise Exception("Requested provider is not configured {}".format(self.default_provider))

    def _resolve_alias(self, templates):
        ret = {}
        for key, value in templates.iteritems():
            if value['type'] == 'alias':
                ret[key] = deepcopy(templates[value['lookup']])
                value.pop('type')
                value.pop('lookup')
                ret[key].update(value)
            else:
                ret[key] = value
        return ret

    def _load_service_templates(self, builtIn):
        # type: (str) -> dict
        with open(builtIn) as builtin:
            return json.load(builtin)

    def get_default_code_creator(self):
        # type: () -> CodeCreator
        return self.code_creators[self.default_provider]

    def create_project(self, service_definition, app_dir, extra_config=None):
        # type: (Service, str) -> None
        project = self.get_default_code_creator().create_project(service_definition=service_definition,
                                                                 app_dir=app_dir,extra_config=extra_config)
        service_type_ = self.templates[service_definition.get_service_type()]
        create_service_def = service_type_.get('generate-service-definition', True)
        if create_service_def:
           self.service_template_generator.create_project(service_definition,app_dir,service_type=service_type_.get('service-definition',None))
        return project



import os


# noinspection PyUnresolvedReferences
from infra_buddy.commandline import cli
from infra_buddy.commands.generate_service_definition import command as gen_service
from infra_buddy.context.deploy_ctx import DeployContext

from service_buddy.service.loader import safe_mkdir


class ServiceTemplateGenerator(object):
    def create_project(self, service_definition, app_dir, service_type=None):
        if not service_type:
            service_type = service_definition.get_service_type()
        service_def_folder = os.path.join(service_definition.get_service_directory(app_dir), "service")
        safe_mkdir(service_def_folder)
        gen_service.do_command(DeployContext.create_deploy_context(
            application=service_definition.get_app(),
            role=service_definition.get_role(),
            environment='dev'
        ), service_type=service_type,
            destination=service_def_folder)



class Application(dict):
    def __init__(self, application):
        super(Application, self).__init__()
        self.application = application

    def add_service(self, role, service):
        self[role] = service

    def get_contract_test_git_url(self):
        for name,definition in self.iteritems():
            if definition.get_service_type() == 'contract-tests':
                return definition.get_git_url()


import click
from service_buddy.commandline import cli


@cli.command(name='clone',short_help="Clone all of the existing service definition repos onto the local file system")
@click.pass_obj
def git_clone(service_ctx):
    service_ctx.vcs.validate_repositories(service_ctx.application_map)
    service_ctx.vcs.clone_service(service_ctx.application_map, service_ctx.destination_directory)



import click

from service_buddy.commandline import cli

from service_buddy.context.service_context import ServiceContext
from service_buddy.util.pretty_printer import pretty_print_services


@cli.command(name='list', short_help="Print definitions for services.")
@click.option('--validate-repository', envvar='VALIDATE',  is_flag=True,
              help='Validate existence of repository in VCS.')
@click.pass_obj
def git_exec(service_ctx, validate_repository):
    # type: (ServiceContext, str,str) -> None
    application_map = service_ctx.application_map
    if validate_repository:
        service_ctx.vcs.validate_repositories(application_map)
    pretty_print_services(application_map)


#!/usr/bin/env python

from setuptools import setup
from setuptools.command.install import install as _install

class install(_install):
    def pre_install_script(self):
        pass

    def post_install_script(self):
        pass

    def run(self):
        self.pre_install_script()

        _install.run(self)

        self.post_install_script()

if __name__ == '__main__':
    setup(
        name = 'service-buddy',
        version = '0.1.79',
        description = 'CLI for managing micro-services',
        long_description = 'CLI for managing micro-services',
        author = '',
        author_email = '',
        license = 'Apache 2.0',
        url = 'https://github.com/AlienVault-Engineering/service-buddy',
        scripts = ['scripts/service-buddy'],
        packages = [
            'service_buddy',
            'service_buddy.context',
            'service_buddy.ci',
            'service_buddy.service',
            'service_buddy.code',
            'service_buddy.util',
            'service_buddy.commands',
            'service_buddy.vcs',
            'service_buddy.commands.initialize',
            'service_buddy.commands.bootstrap',
            'service_buddy.commands.list',
            'service_buddy.commands.git',
            'service_buddy.commands.clone'
        ],
        namespace_packages = [],
        py_modules = [],
        classifiers = [
            'Development Status :: 3 - Alpha',
            'Programming Language :: Python'
        ],
        entry_points = {},
        data_files = [],
        package_data = {
            'service_buddy': ['code/builtin-code-templates.json']
        },
        install_requires = [
            'PyGithub',
            'cookiecutter',
            'pybitbucket',
            'click',
            'infra_buddy'
        ],
        dependency_links = [],
        zip_safe = True,
        cmdclass = {'install': install},
        keywords = '',
        python_requires = '',
        obsoletes = [],
    )



import os

import click
from click import Choice

from infra_buddy.context.deploy_ctx import DeployContext
from service_buddy.ci import ci

from service_buddy.code.code_creator import CodeCreator
from service_buddy.commandline import cli
from service_buddy.context.service_context import ServiceContext
from service_buddy.service.service import Service
from infra_buddy.commands.bootstrap import command as bootstrap
from service_buddy.vcs import vcs


@cli.command(name='bootstrap', short_help="Create a repository containing the initial definition for a micro-service "
                                          "stack managed by service-buddy.")
@click.option("--application", envvar='APPLICATION', required=True,
              help='Application name for new micro-service stack.  Brevity suggested.')
@click.pass_obj
def bootstrap(service_ctx, application):
    # type: (ServiceContext, str,bool,str) -> None
    cc = CodeCreator("", service_ctx.dry_run)
    service_def = Service(app=application, role="master", definition={})
    service_def.set_service_type("service-buddy-master")
    directory = service_ctx.destination_directory
    if directory == "./code": #override default for this usecase
        directory = "./"
    config = {}
    vcs_provider = click.prompt('Please select your source code repository', type=Choice(vcs.vcs_providers) )
    config['vcs-provider'] = vcs_provider
    vcs_options = vcs.options
    for key,value in vcs_options.iteritems():
        config['vcs_{}'.format(key)] = click.prompt("VCS: {} ({})".format(value,key))
    ci_provider = click.prompt('Please select your build system - ', type=Choice(ci.build_systems) )
    config['build-system-provider'] = ci_provider
    ci_options = ci.build_system_map[ci_provider].options()
    for key,value in ci_options.iteritems():
        config['build-system-{}'.format(key)] = click.prompt("Build System: {} ({})".format(value,key))
    cc.create_project(service_definition=service_def, app_dir=directory,extra_config=config)
    



import click

from service_buddy.commandline import cli
from service_buddy.context.service_context import ServiceContext
from service_buddy.service.initializer import Initializer


@cli.command(name='init',short_help="Analyze service definitions and initialize any new services.")
@click.option('--code-template-definitions',
              envvar='CODE_TEMPLATE_DIR',
              type=click.Path(exists=True),
              required=True,
              default="./code-templates",
              help='File containing references to custom service templates.  '
                   ' JSON dictionary {service-type: {type: file|git, location: relative path | git URL}}')
@click.option('--skip-build-creation', envvar='SKIP_BUILD_CREATE',  is_flag=True, default=False,
              help='Skip the creation of build step.')
@click.option('--force-build-creation', envvar='FORCE_BUILD_CREATE',  is_flag=True, default=False,
              help='Force the creation of build step.')
@click.option('--skip-git-creation', envvar='SKIP_GIT_CREATE',  is_flag=True, default=False,
              help='Skip the creation of git repository.')
@click.option('--skip-code-creation', envvar='SKIP_CODE_CREATE',  is_flag=True, default=False,
              help='Skip the templating of the source code.')
@click.pass_obj
def list_service(service_ctx,code_template_definitions, skip_build_creation, force_build_creation, skip_code_creation,skip_git_creation):
    # type: (ServiceContext, str,str) -> None
    vcs =service_ctx.vcs
    application_map = service_ctx.application_map
    vcs.validate_repositories(application_map)
    init = Initializer(vcs=vcs,
                       destination_directory=service_ctx.destination_directory,
                       dry_run=service_ctx.dry_run,
                       code_template_directory=code_template_definitions,
                       skip_code_creation=skip_code_creation,
                       skip_build_creation=skip_build_creation,
                       force_build_creation=force_build_creation,
                       skip_git_creation=skip_git_creation,
                       )
    init.initialize_services(application_map)
